@@ -1,4 +1,4 @@
-/* 
+/*
  * threadPoolCmd.c --
  *
  * This file implements the Tcl thread pools.
@@ -10,7 +10,7 @@
  * ----------------------------------------------------------------------------
  */
 
-#include "tclThread.h"
+#include "tclThreadInt.h"
 
 /*
  * Structure to maintain idle poster threads
@@ -27,7 +27,7 @@ typedef struct TpoolWaiter {
  */
 
 typedef struct ThreadPool {
-    unsigned int jobId;             /* Job counter */
+    Tcl_WideInt jobId;              /* Job counter */
     int idleTime;                   /* Time in secs a worker thread idles */
     int tearDown;                   /* Set to 1 to tear down the pool */
     int suspend;                    /* Set to 1 to suspend pool processing */
@@ -60,9 +60,9 @@ typedef struct ThreadPool {
 
 typedef struct TpoolResult {
     int detached;                   /* Result is to be ignored */
-    unsigned int jobId;             /* The job id of the current job */
+    Tcl_WideInt jobId;              /* The job id of the current job */
     char *script;                   /* Script to evaluate in worker thread */
-    int scriptLen;                  /* Length of the script */    
+    int scriptLen;                  /* Length of the script */
     int retcode;                    /* Tcl return code of the current job */
     char *result;                   /* Tcl result of the current job */
     char *errorCode;                /* On error: content of the errorCode */
@@ -118,61 +118,61 @@ static Tcl_ObjCmdProc TpoolNamesObjCmd;
  */
 
 static int
-CreateWorker   _ANSI_ARGS_((Tcl_Interp *interp, ThreadPool *tpoolPtr));
+CreateWorker(Tcl_Interp *interp, ThreadPool *tpoolPtr);
 
 static Tcl_ThreadCreateType
-TpoolWorker    _ANSI_ARGS_((ClientData clientData));
+TpoolWorker(ClientData clientData);
 
 static int
-RunStopEvent   _ANSI_ARGS_((Tcl_Event *evPtr, int mask));
+RunStopEvent(Tcl_Event *evPtr, int mask);
 
 static void
-PushWork       _ANSI_ARGS_((TpoolResult *rPtr, ThreadPool *tpoolPtr));
+PushWork(TpoolResult *rPtr, ThreadPool *tpoolPtr);
 
 static TpoolResult*
-PopWork        _ANSI_ARGS_((ThreadPool *tpoolPtr));
+PopWork(ThreadPool *tpoolPtr);
 
 static void
-PushWaiter     _ANSI_ARGS_((ThreadPool *tpoolPtr));
+PushWaiter(ThreadPool *tpoolPtr);
 
 static TpoolWaiter*
-PopWaiter      _ANSI_ARGS_((ThreadPool *tpoolPtr));
+PopWaiter(ThreadPool *tpoolPtr);
 
 static void
-SignalWaiter   _ANSI_ARGS_((ThreadPool *tpoolPtr));
+SignalWaiter(ThreadPool *tpoolPtr);
 
 static int
-TpoolEval      _ANSI_ARGS_((Tcl_Interp *interp, char *script, int scriptLen,
-                            TpoolResult *rPtr));
+TpoolEval(Tcl_Interp *interp, char *script, int scriptLen,
+                            TpoolResult *rPtr);
 static void
-SetResult      _ANSI_ARGS_((Tcl_Interp *interp, TpoolResult *rPtr));
+SetResult(Tcl_Interp *interp, TpoolResult *rPtr);
 
-static ThreadPool* 
-GetTpool       _ANSI_ARGS_((const char *tpoolName));
+static ThreadPool*
+GetTpool(const char *tpoolName);
 
-static ThreadPool* 
-GetTpoolUnl    _ANSI_ARGS_((const char *tpoolName));
+static ThreadPool*
+GetTpoolUnl(const char *tpoolName);
 
 static void
-ThrExitHandler _ANSI_ARGS_((ClientData clientData));
+ThrExitHandler(ClientData clientData);
 
 static void
-AppExitHandler _ANSI_ARGS_((ClientData clientData));
+AppExitHandler(ClientData clientData);
 
 static int
-TpoolReserve   _ANSI_ARGS_((ThreadPool *tpoolPtr));
+TpoolReserve(ThreadPool *tpoolPtr);
 
 static int
-TpoolRelease   _ANSI_ARGS_((ThreadPool *tpoolPtr));
+TpoolRelease(ThreadPool *tpoolPtr);
 
 static void
-TpoolSuspend   _ANSI_ARGS_((ThreadPool *tpoolPtr));
+TpoolSuspend(ThreadPool *tpoolPtr);
 
 static void
-TpoolResume   _ANSI_ARGS_((ThreadPool *tpoolPtr));
+TpoolResume(ThreadPool *tpoolPtr);
 
 static void
-InitWaiter     _ANSI_ARGS_((void));
+InitWaiter(void);
 
 
 /*
@@ -180,7 +180,7 @@ InitWaiter     _ANSI_ARGS_((void));
  *
  * TpoolCreateObjCmd --
  *
- *  This procedure is invoked to process the "tpool::create" Tcl 
+ *  This procedure is invoked to process the "tpool::create" Tcl
  *  command. See the user documentation for details on what it does.
  *
  * Results:
@@ -199,11 +199,11 @@ TpoolCreateObjCmd(dummy, interp, objc, objv)
     int         objc;           /* Number of arguments. */
     Tcl_Obj    *const objv[];   /* Argument objects. */
 {
-    int ii, minw, maxw, idle, len;
+    int ii, minw, maxw, idle;
     char buf[64], *exs = NULL, *cmd = NULL;
     ThreadPool *tpoolPtr;
 
-    /* 
+    /*
      * Syntax:  tpool::create ?-minworkers count?
      *                        ?-maxworkers count?
      *                        ?-initcmd script?
@@ -238,11 +238,11 @@ TpoolCreateObjCmd(dummy, interp, objc, objv)
                 return TCL_ERROR;
             }
         } else if (OPT_CMP(opt, "-initcmd")) {
-            const char *val = Tcl_GetStringFromObj(objv[ii+1], &len);
-            cmd  = strcpy(Tcl_Alloc(len+1), val);
+            const char *val = Tcl_GetString(objv[ii+1]);
+            cmd  = strcpy(ckalloc(objv[ii+1]->length+1), val);
         } else if (OPT_CMP(opt, "-exitcmd")) {
-            const char *val = Tcl_GetStringFromObj(objv[ii+1], &len);
-            exs  = strcpy(Tcl_Alloc(len+1), val);
+            const char *val = Tcl_GetString(objv[ii+1]);
+            exs  = strcpy(ckalloc(objv[ii+1]->length+1), val);
         } else {
             goto usage;
         }
@@ -266,7 +266,7 @@ TpoolCreateObjCmd(dummy, interp, objc, objv)
      * Allocate and initialize thread pool structure
      */
 
-    tpoolPtr = (ThreadPool*)Tcl_Alloc(sizeof(ThreadPool));
+    tpoolPtr = (ThreadPool*)ckalloc(sizeof(ThreadPool));
     memset(tpoolPtr, 0, sizeof(ThreadPool));
 
     tpoolPtr->minWorkers  = minw;
@@ -316,7 +316,7 @@ TpoolCreateObjCmd(dummy, interp, objc, objv)
  *
  * TpoolPostObjCmd --
  *
- *  This procedure is invoked to process the "tpool::post" Tcl 
+ *  This procedure is invoked to process the "tpool::post" Tcl
  *  command. See the user documentation for details on what it does.
  *
  * Results:
@@ -335,15 +335,16 @@ TpoolPostObjCmd(dummy, interp, objc, objv)
     int         objc;           /* Number of arguments. */
     Tcl_Obj    *const objv[];   /* Argument objects. */
 {
-    unsigned int jobId = 0;
-    int ii, detached = 0, nowait = 0, len;
+    Tcl_WideInt jobId = 0;
+    int ii, detached = 0, nowait = 0;
+    size_t len;
     const char *tpoolName, *script;
     TpoolResult *rPtr;
     ThreadPool *tpoolPtr;
 
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
-    /* 
+    /*
      * Syntax: tpool::post ?-detached? ?-nowait? tpoolId script
      */
 
@@ -363,19 +364,28 @@ TpoolPostObjCmd(dummy, interp, objc, objv)
         }
     }
 
+    /*
+     * We expect exactly two arguments remaining after options
+     */
+    if (objc - ii != 2)
+    {
+        goto usage;
+    }
+
     tpoolName = Tcl_GetString(objv[ii]);
-    script    = Tcl_GetStringFromObj(objv[ii+1], &len);
+    script    = Tcl_GetString(objv[ii+1]);
+    len = objv[ii+1]->length;
     tpoolPtr  = GetTpool(tpoolName);
     if (tpoolPtr == NULL) {
-        Tcl_AppendResult(interp, "can not find threadpool \"", tpoolName, 
+        Tcl_AppendResult(interp, "can not find threadpool \"", tpoolName,
                          "\"", NULL);
         return TCL_ERROR;
     }
-    
+
     /*
      * Initialize per-thread private data for this caller
      */
-    
+
     InitWaiter();
 
     /*
@@ -383,25 +393,30 @@ TpoolPostObjCmd(dummy, interp, objc, objv)
      */
 
     Tcl_MutexLock(&tpoolPtr->mutex);
-    if (nowait && tpoolPtr->numWorkers == 0) {
+    if (nowait) {
+        if (tpoolPtr->numWorkers == 0) {
 
-        /*
-         * Do not wait for an idle thread but assure
-         * there is at least one worker started.
-         */
+            /*
+             * Assure there is at least one worker running.
+             */
+
+            PushWaiter(tpoolPtr);
+            if (CreateWorker(interp, tpoolPtr) != TCL_OK) {
+                Tcl_MutexUnlock(&tpoolPtr->mutex);
+                return TCL_ERROR;
+            }
+
+            /*
+             * Wait for worker to start while servicing the event loop
+             */
 
-        PushWaiter(tpoolPtr);
-        if (CreateWorker(interp, tpoolPtr) != TCL_OK) {
             Tcl_MutexUnlock(&tpoolPtr->mutex);
-            return TCL_ERROR;
+            tsdPtr->stop = -1;
+            while(tsdPtr->stop == -1) {
+                Tcl_DoOneEvent(TCL_ALL_EVENTS);
+            }
+            Tcl_MutexLock(&tpoolPtr->mutex);
         }
-        /* Wait for worker to start and service the event loop */
-        Tcl_MutexUnlock(&tpoolPtr->mutex);
-        tsdPtr->stop = -1;
-        while(tsdPtr->stop == -1) {
-            Tcl_DoOneEvent(TCL_ALL_EVENTS);
-        }
-        Tcl_MutexLock(&tpoolPtr->mutex);
     } else {
 
         /*
@@ -413,13 +428,21 @@ TpoolPostObjCmd(dummy, interp, objc, objv)
         while (tpoolPtr->idleWorkers == 0) {
             PushWaiter(tpoolPtr);
             if (tpoolPtr->numWorkers < tpoolPtr->maxWorkers) {
-                /* No more free workers; start new one */
+
+                /*
+                 * No more free workers; start new one
+                 */
+
                 if (CreateWorker(interp, tpoolPtr) != TCL_OK) {
                     Tcl_MutexUnlock(&tpoolPtr->mutex);
                     return TCL_ERROR;
                 }
             }
-            /* Wait for any idle worker and service the event loop */
+
+            /*
+             * Wait for worker to start while servicing the event loop
+             */
+
             Tcl_MutexUnlock(&tpoolPtr->mutex);
             tsdPtr->stop = -1;
             while(tsdPtr->stop == -1) {
@@ -433,7 +456,7 @@ TpoolPostObjCmd(dummy, interp, objc, objv)
      * Create new job ticket and put it on the list.
      */
 
-    rPtr = (TpoolResult*)Tcl_Alloc(sizeof(TpoolResult));
+    rPtr = (TpoolResult*)ckalloc(sizeof(TpoolResult));
     memset(rPtr, 0, sizeof(TpoolResult));
 
     if (detached == 0) {
@@ -441,7 +464,7 @@ TpoolPostObjCmd(dummy, interp, objc, objv)
         rPtr->jobId = jobId;
     }
 
-    rPtr->script    = strcpy(Tcl_Alloc(len+1), script);
+    rPtr->script    = strcpy(ckalloc(len+1), script);
     rPtr->scriptLen = len;
     rPtr->detached  = detached;
     rPtr->threadId  = Tcl_GetCurrentThread();
@@ -451,9 +474,9 @@ TpoolPostObjCmd(dummy, interp, objc, objv)
     Tcl_MutexUnlock(&tpoolPtr->mutex);
 
     if (detached == 0) {
-        Tcl_SetObjResult(interp, Tcl_NewIntObj(jobId));
+        Tcl_SetObjResult(interp, Tcl_NewWideIntObj(jobId));
     }
-    
+
     return TCL_OK;
 
   usage:
@@ -466,7 +489,7 @@ TpoolPostObjCmd(dummy, interp, objc, objv)
  *
  * TpoolWaitObjCmd --
  *
- *  This procedure is invoked to process the "tpool::wait" Tcl 
+ *  This procedure is invoked to process the "tpool::wait" Tcl
  *  command. See the user documentation for details on what it does.
  *
  * Results:
@@ -485,8 +508,9 @@ TpoolWaitObjCmd(dummy, interp, objc, objv)
     Tcl_Obj    *const objv[];   /* Argument objects. */
 {
     int ii, done, wObjc;
-    unsigned int jobId;
-    char *tpoolName, *listVar = NULL;
+    Tcl_WideInt jobId;
+    char *tpoolName;
+    Tcl_Obj *listVar = NULL;
     Tcl_Obj *waitList, *doneList, **wObjv;
     ThreadPool *tpoolPtr;
     TpoolResult *rPtr;
@@ -494,7 +518,7 @@ TpoolWaitObjCmd(dummy, interp, objc, objv)
 
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
-    /* 
+    /*
      * Syntax: tpool::wait tpoolId jobIdList ?listVar?
      */
 
@@ -503,7 +527,7 @@ TpoolWaitObjCmd(dummy, interp, objc, objv)
         return TCL_ERROR;
     }
     if (objc == 4) {
-        listVar = Tcl_GetString(objv[3]);
+        listVar = objv[3];
     }
     if (Tcl_ListObjGetElements(interp, objv[2], &wObjc, &wObjv) != TCL_OK) {
         return TCL_ERROR;
@@ -524,28 +548,22 @@ TpoolWaitObjCmd(dummy, interp, objc, objv)
     while (1) {
         waitList = Tcl_NewListObj(0, NULL);
         for (ii = 0; ii < wObjc; ii++) {
-            if (Tcl_GetIntFromObj(interp, wObjv[ii], (int *)&jobId) != TCL_OK) {
+            if (Tcl_GetWideIntFromObj(interp, wObjv[ii], &jobId) != TCL_OK) {
                 Tcl_MutexUnlock(&tpoolPtr->mutex);
                 return TCL_ERROR;
             }
-            hPtr = Tcl_FindHashEntry(&tpoolPtr->jobsDone, (char*)jobId);
+            hPtr = Tcl_FindHashEntry(&tpoolPtr->jobsDone, (void *)(size_t)jobId);
             if (hPtr) {
                 rPtr = (TpoolResult*)Tcl_GetHashValue(hPtr);
             } else {
-                for (rPtr = tpoolPtr->workHead; rPtr; rPtr = rPtr->nextPtr) {
-                    if (rPtr->jobId == jobId) {
-                        break;
-                    }
-                }
-                if (rPtr == NULL) {
-                    continue; /* Bogus job id; ignore */
-                }
+                rPtr = NULL;
             }
-            if (rPtr->detached) {
-                continue; /* A detached job */
-            }
-            if (rPtr->result) {
-                done++; /* Job has been processed */
+            if (rPtr == NULL) {
+                if (listVar) {
+                    Tcl_ListObjAppendElement(interp, waitList, wObjv[ii]);
+                }
+            } else if (!rPtr->detached && rPtr->result) {
+                done++;
                 Tcl_ListObjAppendElement(interp, doneList, wObjv[ii]);
             } else if (listVar) {
                 Tcl_ListObjAppendElement(interp, waitList, wObjv[ii]);
@@ -560,7 +578,7 @@ TpoolWaitObjCmd(dummy, interp, objc, objv)
          * of the next job and try again.
          */
 
-        Tcl_DecrRefCount(waitList); 
+        Tcl_DecrRefCount(waitList);
         PushWaiter(tpoolPtr);
 
         Tcl_MutexUnlock(&tpoolPtr->mutex);
@@ -573,7 +591,7 @@ TpoolWaitObjCmd(dummy, interp, objc, objv)
     Tcl_MutexUnlock(&tpoolPtr->mutex);
 
     if (listVar) {
-        Tcl_SetVar2Ex(interp, listVar, NULL, waitList, 0);
+        Tcl_ObjSetVar2(interp, listVar, NULL, waitList, 0);
     }
 
     Tcl_SetObjResult(interp, doneList);
@@ -586,7 +604,7 @@ TpoolWaitObjCmd(dummy, interp, objc, objv)
  *
  * TpoolCancelObjCmd --
  *
- *  This procedure is invoked to process the "tpool::cancel" Tcl 
+ *  This procedure is invoked to process the "tpool::cancel" Tcl
  *  command. See the user documentation for details on what it does.
  *
  * Results:
@@ -604,14 +622,16 @@ TpoolCancelObjCmd(dummy, interp, objc, objv)
     int         objc;           /* Number of arguments. */
     Tcl_Obj    *const objv[];   /* Argument objects. */
 {
-    int ii, wObjc, jobId;
-    char *tpoolName, *listVar = NULL;
+    int ii, wObjc;
+    Tcl_WideInt jobId;
+    char *tpoolName;
+    Tcl_Obj *listVar = NULL;
     Tcl_Obj *doneList, *waitList, **wObjv;
     ThreadPool *tpoolPtr;
     TpoolResult *rPtr;
 
-    /* 
-     * Syntax: tpool::wait tpoolId jobIdList ?listVar?
+    /*
+     * Syntax: tpool::cancel tpoolId jobIdList ?listVar?
      */
 
     if (objc < 3 || objc > 4) {
@@ -619,7 +639,7 @@ TpoolCancelObjCmd(dummy, interp, objc, objv)
         return TCL_ERROR;
     }
     if (objc == 4) {
-        listVar = Tcl_GetString(objv[3]);
+        listVar = objv[3];
     }
     if (Tcl_ListObjGetElements(interp, objv[2], &wObjc, &wObjv) != TCL_OK) {
         return TCL_ERROR;
@@ -631,18 +651,18 @@ TpoolCancelObjCmd(dummy, interp, objc, objv)
                          "\"", NULL);
         return TCL_ERROR;
     }
- 
+
     InitWaiter();
     doneList = Tcl_NewListObj(0, NULL);
     waitList = Tcl_NewListObj(0, NULL);
 
     Tcl_MutexLock(&tpoolPtr->mutex);
     for (ii = 0; ii < wObjc; ii++) {
-        if (Tcl_GetIntFromObj(interp, wObjv[ii], &jobId) != TCL_OK) {
+        if (Tcl_GetWideIntFromObj(interp, wObjv[ii], &jobId) != TCL_OK) {
             return TCL_ERROR;
         }
         for (rPtr = tpoolPtr->workHead; rPtr; rPtr = rPtr->nextPtr) {
-            if (rPtr->jobId == (unsigned int)jobId) {
+            if (rPtr->jobId == jobId) {
                 if (rPtr->prevPtr != NULL) {
                     rPtr->prevPtr->nextPtr = rPtr->nextPtr;
                 } else {
@@ -654,8 +674,8 @@ TpoolCancelObjCmd(dummy, interp, objc, objv)
                     tpoolPtr->workTail = rPtr->prevPtr;
                 }
                 SetResult(NULL, rPtr); /* Just to free the result */
-                Tcl_Free(rPtr->script);
-                Tcl_Free((char*)rPtr);
+                ckfree(rPtr->script);
+                ckfree((char*)rPtr);
                 Tcl_ListObjAppendElement(interp, doneList, wObjv[ii]);
                 break;
             }
@@ -667,7 +687,7 @@ TpoolCancelObjCmd(dummy, interp, objc, objv)
     Tcl_MutexUnlock(&tpoolPtr->mutex);
 
     if (listVar) {
-        Tcl_SetVar2Ex(interp, listVar, NULL, waitList, 0);
+        Tcl_ObjSetVar2(interp, listVar, NULL, waitList, 0);
     }
 
     Tcl_SetObjResult(interp, doneList);
@@ -680,7 +700,7 @@ TpoolCancelObjCmd(dummy, interp, objc, objv)
  *
  * TpoolGetObjCmd --
  *
- *  This procedure is invoked to process the "tpool::get" Tcl 
+ *  This procedure is invoked to process the "tpool::get" Tcl
  *  command. See the user documentation for details on what it does.
  *
  * Results:
@@ -698,13 +718,15 @@ TpoolGetObjCmd(dummy, interp, objc, objv)
     int         objc;           /* Number of arguments. */
     Tcl_Obj    *const objv[];   /* Argument objects. */
 {
-    int ret, jobId;
-    char *tpoolName, *resVar = NULL;
+    int ret;
+    Tcl_WideInt jobId;
+    char *tpoolName;
+    Tcl_Obj *resVar = NULL;
     ThreadPool *tpoolPtr;
     TpoolResult *rPtr;
     Tcl_HashEntry *hPtr;
 
-    /* 
+    /*
      * Syntax: tpool::get tpoolId jobId ?result?
      */
 
@@ -712,11 +734,11 @@ TpoolGetObjCmd(dummy, interp, objc, objv)
         Tcl_WrongNumArgs(interp, 1, objv, "tpoolId jobId ?result?");
         return TCL_ERROR;
     }
-    if (Tcl_GetIntFromObj(interp, objv[2], &jobId) != TCL_OK) {
+    if (Tcl_GetWideIntFromObj(interp, objv[2], &jobId) != TCL_OK) {
         return TCL_ERROR;
     }
     if (objc == 4) {
-        resVar = Tcl_GetString(objv[3]);
+        resVar = objv[3];
     }
 
     /*
@@ -726,7 +748,7 @@ TpoolGetObjCmd(dummy, interp, objc, objv)
     tpoolName = Tcl_GetString(objv[1]);
     tpoolPtr  = GetTpool(tpoolName);
     if (tpoolPtr == NULL) {
-        Tcl_AppendResult(interp, "can not find threadpool \"", tpoolName, 
+        Tcl_AppendResult(interp, "can not find threadpool \"", tpoolName,
                          "\"", NULL);
         return TCL_ERROR;
     }
@@ -738,7 +760,7 @@ TpoolGetObjCmd(dummy, interp, objc, objv)
      */
 
     Tcl_MutexLock(&tpoolPtr->mutex);
-    hPtr = Tcl_FindHashEntry(&tpoolPtr->jobsDone, (char*)jobId);
+    hPtr = Tcl_FindHashEntry(&tpoolPtr->jobsDone, (void *)(size_t)jobId);
     if (hPtr == NULL) {
         Tcl_MutexUnlock(&tpoolPtr->mutex);
         Tcl_AppendResult(interp, "no such job", NULL);
@@ -756,12 +778,12 @@ TpoolGetObjCmd(dummy, interp, objc, objv)
 
     ret = rPtr->retcode;
     SetResult(interp, rPtr);
-    Tcl_Free((char*)rPtr);
+    ckfree((char*)rPtr);
 
     if (resVar) {
-        Tcl_SetVar2Ex(interp, resVar, NULL, Tcl_GetObjResult(interp), 0);
+        Tcl_ObjSetVar2(interp, resVar, NULL, Tcl_GetObjResult(interp), 0);
         Tcl_SetObjResult(interp, Tcl_NewIntObj(ret));
-        ret = TCL_OK; 
+        ret = TCL_OK;
     }
 
     return ret;
@@ -772,7 +794,7 @@ TpoolGetObjCmd(dummy, interp, objc, objv)
  *
  * TpoolReserveObjCmd --
  *
- *  This procedure is invoked to process the "tpool::preserve" Tcl 
+ *  This procedure is invoked to process the "tpool::preserve" Tcl
  *  command. See the user documentation for details on what it does.
  *
  * Results:
@@ -810,16 +832,16 @@ TpoolReserveObjCmd(dummy, interp, objc, objv)
     tpoolPtr  = GetTpoolUnl(tpoolName);
     if (tpoolPtr == NULL) {
         Tcl_MutexUnlock(&listMutex);
-        Tcl_AppendResult(interp, "can not find threadpool \"", tpoolName, 
+        Tcl_AppendResult(interp, "can not find threadpool \"", tpoolName,
                          "\"", NULL);
         return TCL_ERROR;
     }
 
-    ret = TpoolReserve(tpoolPtr); 
+    ret = TpoolReserve(tpoolPtr);
     Tcl_MutexUnlock(&listMutex);
     Tcl_SetObjResult(interp, Tcl_NewIntObj(ret));
 
-    return TCL_OK; 
+    return TCL_OK;
 }
 
 /*
@@ -827,7 +849,7 @@ TpoolReserveObjCmd(dummy, interp, objc, objv)
  *
  * TpoolReleaseObjCmd --
  *
- *  This procedure is invoked to process the "tpool::release" Tcl 
+ *  This procedure is invoked to process the "tpool::release" Tcl
  *  command. See the user documentation for details on what it does.
  *
  * Results:
@@ -870,11 +892,11 @@ TpoolReleaseObjCmd(dummy, interp, objc, objv)
         return TCL_ERROR;
     }
 
-    ret = TpoolRelease(tpoolPtr); 
+    ret = TpoolRelease(tpoolPtr);
     Tcl_MutexUnlock(&listMutex);
     Tcl_SetObjResult(interp, Tcl_NewIntObj(ret));
 
-    return TCL_OK; 
+    return TCL_OK;
 }
 
 /*
@@ -882,7 +904,7 @@ TpoolReleaseObjCmd(dummy, interp, objc, objv)
  *
  * TpoolSuspendObjCmd --
  *
- *  This procedure is invoked to process the "tpool::suspend" Tcl 
+ *  This procedure is invoked to process the "tpool::suspend" Tcl
  *  command. See the user documentation for details on what it does.
  *
  * Results:
@@ -917,14 +939,14 @@ TpoolSuspendObjCmd(dummy, interp, objc, objv)
     tpoolPtr  = GetTpool(tpoolName);
 
     if (tpoolPtr == NULL) {
-        Tcl_AppendResult(interp, "can not find threadpool \"", tpoolName, 
+        Tcl_AppendResult(interp, "can not find threadpool \"", tpoolName,
                          "\"", NULL);
         return TCL_ERROR;
     }
 
-    TpoolSuspend(tpoolPtr); 
+    TpoolSuspend(tpoolPtr);
 
-    return TCL_OK; 
+    return TCL_OK;
 }
 
 /*
@@ -932,7 +954,7 @@ TpoolSuspendObjCmd(dummy, interp, objc, objv)
  *
  * TpoolResumeObjCmd --
  *
- *  This procedure is invoked to process the "tpool::resume" Tcl 
+ *  This procedure is invoked to process the "tpool::resume" Tcl
  *  command. See the user documentation for details on what it does.
  *
  * Results:
@@ -967,14 +989,14 @@ TpoolResumeObjCmd(dummy, interp, objc, objv)
     tpoolPtr  = GetTpool(tpoolName);
 
     if (tpoolPtr == NULL) {
-        Tcl_AppendResult(interp, "can not find threadpool \"", tpoolName, 
+        Tcl_AppendResult(interp, "can not find threadpool \"", tpoolName,
                          "\"", NULL);
         return TCL_ERROR;
     }
 
-    TpoolResume(tpoolPtr); 
+    TpoolResume(tpoolPtr);
 
-    return TCL_OK; 
+    return TCL_OK;
 }
 
 /*
@@ -982,7 +1004,7 @@ TpoolResumeObjCmd(dummy, interp, objc, objv)
  *
  * TpoolNamesObjCmd --
  *
- *  This procedure is invoked to process the "tpool::names" Tcl 
+ *  This procedure is invoked to process the "tpool::names" Tcl
  *  command. See the user documentation for details on what it does.
  *
  * Results:
@@ -1003,7 +1025,7 @@ TpoolNamesObjCmd(dummy, interp, objc, objv)
 {
     ThreadPool *tpoolPtr;
     Tcl_Obj *listObj = Tcl_NewListObj(0, NULL);
-    
+
     Tcl_MutexLock(&listMutex);
     for (tpoolPtr = tpoolList; tpoolPtr; tpoolPtr = tpoolPtr->nextPtr) {
         char buf[32];
@@ -1051,14 +1073,14 @@ CreateWorker(interp, tpoolPtr)
     result.tpoolPtr = tpoolPtr;
 
     /*
-     * Create new worker thread here. Wait for the thread to start 
+     * Create new worker thread here. Wait for the thread to start
      * because it's using the ThreadResult arg which is on our stack.
      */
 
     Tcl_MutexLock(&startMutex);
     if (Tcl_CreateThread(&id, TpoolWorker, (ClientData)&result,
                          TCL_THREAD_STACK_DEFAULT, 0) != TCL_OK) {
-        Tcl_SetResult(interp, "can't create a new thread", TCL_STATIC);
+        Tcl_SetObjResult(interp, Tcl_NewStringObj("can't create a new thread", -1));
         Tcl_MutexUnlock(&startMutex);
         return TCL_ERROR;
     }
@@ -1071,7 +1093,7 @@ CreateWorker(interp, tpoolPtr)
      * Set error-related information if the thread
      * failed to initialize correctly.
      */
-    
+
     if (result.retcode == 1) {
         result.retcode = TCL_ERROR;
         SetResult(interp, &result);
@@ -1100,14 +1122,14 @@ CreateWorker(interp, tpoolPtr)
 static Tcl_ThreadCreateType
 TpoolWorker(clientData)
     ClientData clientData;
-{    
+{
     TpoolResult          *rPtr = (TpoolResult*)clientData;
     ThreadPool       *tpoolPtr = rPtr->tpoolPtr;
 
     int tout = 0;
     Tcl_Interp *interp;
     Tcl_Time waitTime, *idlePtr;
-    char *errMsg;
+    const char *errMsg;
 
     Tcl_MutexLock(&startMutex);
 
@@ -1128,10 +1150,10 @@ TpoolWorker(clientData)
         rPtr->retcode = 0;
     }
 #endif
-    
+
     if (rPtr->retcode == 1) {
-        errMsg = (char*)Tcl_GetStringResult(interp);
-        rPtr->result = strcpy(Tcl_Alloc(strlen(errMsg)+1), errMsg);
+        errMsg = Tcl_GetString(Tcl_GetObjResult(interp));
+        rPtr->result = strcpy(ckalloc(strlen(errMsg)+1), errMsg);
         Tcl_ConditionNotify(&tpoolPtr->cond);
         Tcl_MutexUnlock(&startMutex);
         goto out;
@@ -1145,8 +1167,8 @@ TpoolWorker(clientData)
         TpoolEval(interp, tpoolPtr->initScript, -1, rPtr);
         if (rPtr->retcode != TCL_OK) {
             rPtr->retcode = 1;
-            errMsg = (char*)Tcl_GetStringResult(interp);
-            rPtr->result  = strcpy(Tcl_Alloc(strlen(errMsg)+1), errMsg);
+            errMsg = Tcl_GetString(Tcl_GetObjResult(interp));
+            rPtr->result  = strcpy(ckalloc(strlen(errMsg)+1), errMsg);
             Tcl_ConditionNotify(&tpoolPtr->cond);
             Tcl_MutexUnlock(&startMutex);
             goto out;
@@ -1169,7 +1191,7 @@ TpoolWorker(clientData)
      * Tell caller we've started
      */
 
-    tpoolPtr->numWorkers++; 
+    tpoolPtr->numWorkers++;
     Tcl_ConditionNotify(&tpoolPtr->cond);
     Tcl_MutexUnlock(&startMutex);
 
@@ -1214,15 +1236,16 @@ TpoolWorker(clientData)
         }
         Tcl_MutexUnlock(&tpoolPtr->mutex);
         TpoolEval(interp, rPtr->script, rPtr->scriptLen, rPtr);
+        ckfree(rPtr->script);
         Tcl_MutexLock(&tpoolPtr->mutex);
-        Tcl_Free(rPtr->script);
         if (!rPtr->detached) {
             int new;
-            Tcl_SetHashValue(Tcl_CreateHashEntry(&tpoolPtr->jobsDone, 
-                                                 (char*)rPtr->jobId, &new), 
+            Tcl_SetHashValue(Tcl_CreateHashEntry(&tpoolPtr->jobsDone,
+                                                 (void *)(size_t)rPtr->jobId, &new),
                              (ClientData)rPtr);
+            SignalWaiter(tpoolPtr);
         } else {
-            Tcl_Free((char*)rPtr);
+            ckfree((char*)rPtr);
         }
     }
 
@@ -1262,13 +1285,13 @@ TpoolWorker(clientData)
  *  1 (always)
  *
  * Side effects:
- *  None. 
+ *  None.
  *
  *----------------------------------------------------------------------
  */
 static int
 RunStopEvent(eventPtr, mask)
-    Tcl_Event *eventPtr; 
+    Tcl_Event *eventPtr;
     int mask;
 {
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
@@ -1323,7 +1346,7 @@ PushWork(rPtr, tpoolPtr)
 static TpoolResult *
 PopWork(tpoolPtr)
     ThreadPool *tpoolPtr;
-{   
+{
     TpoolResult *rPtr = tpoolPtr->workTail;
 
     if (rPtr == NULL) {
@@ -1364,7 +1387,7 @@ PushWaiter(tpoolPtr)
     if (tpoolPtr->waitTail == NULL) {
         tpoolPtr->waitTail = tsdPtr->waitPtr;
     }
-} 
+}
 
 /*
  *----------------------------------------------------------------------
@@ -1403,24 +1426,24 @@ PopWaiter(tpoolPtr)
 /*
  *----------------------------------------------------------------------
  *
- * GetTpool 
+ * GetTpool
  *
  *  Parses the Tcl threadpool handle and locates the
- *  corresponding threadpool maintenance structure. 
+ *  corresponding threadpool maintenance structure.
  *
  * Results:
- *  Pointer to the threadpool struct or NULL if none found, 
+ *  Pointer to the threadpool struct or NULL if none found,
  *
  * Side effects:
  *  None.
  *
  *----------------------------------------------------------------------
  */
-static ThreadPool* 
-GetTpool(tpoolName) 
+static ThreadPool*
+GetTpool(tpoolName)
     const char *tpoolName;
 {
-    ThreadPool *tpoolPtr; 
+    ThreadPool *tpoolPtr;
 
     Tcl_MutexLock(&listMutex);
     tpoolPtr = GetTpoolUnl(tpoolName);
@@ -1432,14 +1455,14 @@ GetTpool(tpoolName)
 /*
  *----------------------------------------------------------------------
  *
- * GetTpoolUnl 
+ * GetTpoolUnl
  *
  *  Parses the threadpool handle and locates the
- *  corresponding threadpool maintenance structure. 
+ *  corresponding threadpool maintenance structure.
  *  Assumes caller holds the listMutex,
  *
  * Results:
- *  Pointer to the threadpool struct or NULL if none found, 
+ *  Pointer to the threadpool struct or NULL if none found,
  *
  * Side effects:
  *  None.
@@ -1447,8 +1470,8 @@ GetTpool(tpoolName)
  *----------------------------------------------------------------------
  */
 
-static ThreadPool* 
-GetTpoolUnl (tpoolName) 
+static ThreadPool*
+GetTpoolUnl (tpoolName)
     const char *tpoolName;
 {
     ThreadPool *tpool;
@@ -1469,12 +1492,12 @@ GetTpoolUnl (tpoolName)
 /*
  *----------------------------------------------------------------------
  *
- * TpoolEval 
+ * TpoolEval
  *
- *  Evaluates the script and fills in the result structure. 
+ *  Evaluates the script and fills in the result structure.
  *
  * Results:
- *  Standard Tcl result, 
+ *  Standard Tcl result,
  *
  * Side effects:
  *  Many, depending on the script.
@@ -1488,34 +1511,36 @@ TpoolEval(interp, script, scriptLen, rPtr)
     int scriptLen;
     TpoolResult *rPtr;
 {
-    int ret, reslen;
-    char *result, *errorCode, *errorInfo;
-    
+    int ret;
+    size_t reslen;
+    const char *result;
+    const char *errorCode, *errorInfo;
+
     ret = Tcl_EvalEx(interp, script, scriptLen, TCL_EVAL_GLOBAL);
     if (rPtr == NULL || rPtr->detached) {
         return ret;
     }
     rPtr->retcode = ret;
     if (ret == TCL_ERROR) {
-        errorCode = (char*)Tcl_GetVar(interp, "errorCode", TCL_GLOBAL_ONLY);
-        errorInfo = (char*)Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
+        errorCode = Tcl_GetVar2(interp, "errorCode", NULL, TCL_GLOBAL_ONLY);
+        errorInfo = Tcl_GetVar2(interp, "errorInfo", NULL, TCL_GLOBAL_ONLY);
         if (errorCode != NULL) {
-            rPtr->errorCode = Tcl_Alloc(1 + strlen(errorCode));
+            rPtr->errorCode = ckalloc(1 + strlen(errorCode));
             strcpy(rPtr->errorCode, errorCode);
         }
         if (errorInfo != NULL) {
-            rPtr->errorInfo = Tcl_Alloc(1 + strlen(errorInfo));
+            rPtr->errorInfo = ckalloc(1 + strlen(errorInfo));
             strcpy(rPtr->errorInfo, errorInfo);
         }
     }
-    
-    result = (char*)Tcl_GetStringResult(interp);
-    reslen = strlen(result);
-    
+
+    result = Tcl_GetString(Tcl_GetObjResult(interp));
+    reslen = Tcl_GetObjResult(interp)->length;
+
     if (reslen == 0) {
         rPtr->result = threadEmptyResult;
     } else {
-        rPtr->result = strcpy(Tcl_Alloc(1 + reslen), result);
+        rPtr->result = strcpy(ckalloc(1 + reslen), result);
     }
 
     return ret;
@@ -1529,7 +1554,7 @@ TpoolEval(interp, script, scriptLen, rPtr)
  *  Sets the result in current interpreter.
  *
  * Results:
- *  Standard Tcl result, 
+ *  Standard Tcl result,
  *
  * Side effects:
  *  None.
@@ -1541,6 +1566,22 @@ SetResult(interp, rPtr)
     Tcl_Interp *interp;
     TpoolResult *rPtr;
 {
+    if (rPtr->retcode == TCL_ERROR) {
+        if (rPtr->errorCode) {
+            if (interp) {
+                Tcl_SetObjErrorCode(interp,Tcl_NewStringObj(rPtr->errorCode,-1));
+            }
+            ckfree(rPtr->errorCode);
+            rPtr->errorCode = NULL;
+        }
+        if (rPtr->errorInfo) {
+            if (interp) {
+                Tcl_AddErrorInfo(interp, rPtr->errorInfo);
+            }
+            ckfree(rPtr->errorInfo);
+            rPtr->errorInfo = NULL;
+        }
+    }
     if (rPtr->result) {
         if (rPtr->result == threadEmptyResult) {
             if (interp) {
@@ -1550,26 +1591,10 @@ SetResult(interp, rPtr)
             if (interp) {
                 Tcl_SetObjResult(interp, Tcl_NewStringObj(rPtr->result,-1));
             }
-            Tcl_Free(rPtr->result);
+            ckfree(rPtr->result);
             rPtr->result = NULL;
         }
     }
-    if (rPtr->retcode == TCL_ERROR) {
-        if (rPtr->errorCode) {
-            if (interp) {
-                Tcl_SetObjErrorCode(interp,Tcl_NewStringObj(rPtr->errorCode,-1));
-            }
-            Tcl_Free(rPtr->errorCode);
-            rPtr->errorCode = NULL;
-        }
-        if (rPtr->errorInfo) {
-            if (interp) {
-                Tcl_AddObjErrorInfo(interp, rPtr->errorInfo, -1);
-            }
-            Tcl_Free(rPtr->errorInfo);
-            rPtr->errorInfo = NULL;
-        }
-    }
 }
 
 /*
@@ -1577,7 +1602,7 @@ SetResult(interp, rPtr)
  *
  * TpoolReserve --
  *
- *  Does the pool preserve and/or release. Assumes caller holds 
+ *  Does the pool preserve and/or release. Assumes caller holds
  *  the listMutex.
  *
  * Results:
@@ -1600,7 +1625,7 @@ TpoolReserve(tpoolPtr)
  *
  * TpoolRelease --
  *
- *  Does the pool preserve and/or release. Assumes caller holds 
+ *  Does the pool preserve and/or release. Assumes caller holds
  *  the listMutex.
  *
  * Results:
@@ -1620,23 +1645,23 @@ TpoolRelease(tpoolPtr)
     Tcl_HashEntry *hPtr;
     Tcl_HashSearch search;
 
-    if (--tpoolPtr->refCount > 0) { 
+    if (--tpoolPtr->refCount > 0) {
         return tpoolPtr->refCount;
     }
 
     /*
      * Pool is going away; remove from the list of pools,
-     */ 
-    
+     */
+
     SpliceOut(tpoolPtr, tpoolList);
     InitWaiter();
-    
+
     /*
      * Signal and wait for all workers to die.
      */
-    
-    tpoolPtr->tearDown = 1;
+
     Tcl_MutexLock(&tpoolPtr->mutex);
+    tpoolPtr->tearDown = 1;
     while (tpoolPtr->numWorkers > 0) {
         PushWaiter(tpoolPtr);
         Tcl_ConditionNotify(&tpoolPtr->cond);
@@ -1648,16 +1673,16 @@ TpoolRelease(tpoolPtr)
         Tcl_MutexLock(&tpoolPtr->mutex);
     }
     Tcl_MutexUnlock(&tpoolPtr->mutex);
-    
+
     /*
      * Tear down the pool structure
      */
-    
+
     if (tpoolPtr->initScript) {
-        Tcl_Free(tpoolPtr->initScript);
+        ckfree(tpoolPtr->initScript);
     }
     if (tpoolPtr->exitScript) {
-        Tcl_Free(tpoolPtr->exitScript);
+        ckfree(tpoolPtr->exitScript);
     }
 
     /*
@@ -1668,17 +1693,17 @@ TpoolRelease(tpoolPtr)
     while (hPtr != NULL) {
         rPtr = (TpoolResult*)Tcl_GetHashValue(hPtr);
         if (rPtr->result && rPtr->result != threadEmptyResult) {
-            Tcl_Free(rPtr->result);
+            ckfree(rPtr->result);
         }
         if (rPtr->retcode == TCL_ERROR) {
             if (rPtr->errorInfo) {
-                Tcl_Free(rPtr->errorInfo);
+                ckfree(rPtr->errorInfo);
             }
             if (rPtr->errorCode) {
-                Tcl_Free(rPtr->errorCode);
+                ckfree(rPtr->errorCode);
             }
         }
-        Tcl_Free((char*)rPtr);
+        ckfree((char*)rPtr);
         Tcl_DeleteHashEntry(hPtr);
         hPtr = Tcl_NextHashEntry(&search);
     }
@@ -1689,12 +1714,12 @@ TpoolRelease(tpoolPtr)
      */
 
     for (rPtr = tpoolPtr->workHead; rPtr; rPtr = rPtr->nextPtr) {
-        Tcl_Free(rPtr->script);
-        Tcl_Free((char*)rPtr);
+        ckfree(rPtr->script);
+        ckfree((char*)rPtr);
     }
     Tcl_MutexFinalize(&tpoolPtr->mutex);
     Tcl_ConditionFinalize(&tpoolPtr->cond);
-    Tcl_Free((char*)tpoolPtr);
+    ckfree((char*)tpoolPtr);
 
     return 0;
 }
@@ -1705,14 +1730,14 @@ TpoolRelease(tpoolPtr)
  * TpoolSuspend --
  *
  *  Marks the pool as suspended. This prevents pool workers to drain
- *  the pool work queue. 
+ *  the pool work queue.
  *
  * Results:
  *  Value of the suspend flag (1 always).
  *
  * Side effects:
  *  During the suspended state, pool worker threads wlll not timeout
- *  even if the worker inactivity timer has been configured. 
+ *  even if the worker inactivity timer has been configured.
  *
  *----------------------------------------------------------------------
  */
@@ -1731,7 +1756,7 @@ TpoolSuspend(tpoolPtr)
  * TpoolResume --
  *
  *  Clears the pool suspended state. This allows pool workers to drain
- *  the pool work queue again. 
+ *  the pool work queue again.
  *
  * Results:
  *  None.
@@ -1778,7 +1803,7 @@ SignalWaiter(tpoolPtr)
         return;
     }
 
-    evPtr = (Tcl_Event*)Tcl_Alloc(sizeof(Tcl_Event));
+    evPtr = (Tcl_Event*)ckalloc(sizeof(Tcl_Event));
     evPtr->proc = RunStopEvent;
 
     Tcl_ThreadQueueEvent(waitPtr->threadId,(Tcl_Event*)evPtr,TCL_QUEUE_TAIL);
@@ -1806,7 +1831,7 @@ InitWaiter ()
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
     if (tsdPtr->waitPtr == NULL) {
-        tsdPtr->waitPtr = (TpoolWaiter*)Tcl_Alloc(sizeof(TpoolWaiter));
+        tsdPtr->waitPtr = (TpoolWaiter*)ckalloc(sizeof(TpoolWaiter));
         tsdPtr->waitPtr->prevPtr  = NULL;
         tsdPtr->waitPtr->nextPtr  = NULL;
         tsdPtr->waitPtr->threadId = Tcl_GetCurrentThread();
@@ -1835,18 +1860,18 @@ ThrExitHandler(clientData)
 {
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *)clientData;
 
-    Tcl_Free((char*)tsdPtr->waitPtr);
+    ckfree((char*)tsdPtr->waitPtr);
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * AppExitHandler 
+ * AppExitHandler
  *
  *  Deletes all threadpools on application exit.
  *
  * Results:
- *  None. 
+ *  None.
  *
  * Side effects:
  *  None.
@@ -1886,7 +1911,7 @@ AppExitHandler(clientData)
  *----------------------------------------------------------------------
  */
 
-int 
+int
 Tpool_Init (interp)
     Tcl_Interp *interp;                 /* Interp where to create cmds */
 {
